from functools import cached_property
from typing import Final, Optional

from ..logger import log


print = log


class DualAlphabeticController:
    """Контроллер для конвертации между визуально похожими латинскими и кириллическими символами.

    Определяет преобладающий алфавит в строке и конвертирует все DAB-символы (Dual-Alphabet Letters)
    в соответствующие символы выбранного алфавита.
    """

    LATIN_TO_CYRILLIC: Final[dict[str, str]] = {
        'A': 'А', 'B': 'В', 'C': 'С', 'E': 'Е', 'H': 'Н', 'K': 'К',
        'M': 'М', 'O': 'О', 'P': 'Р', 'T': 'Т', 'X': 'Х',
        'a': 'а', 'c': 'с', 'e': 'е', 'o': 'о', 'p': 'р', 'x': 'х',
    }

    CYRILLIC_TO_LATIN: Final[dict[str, str]] = {v: k for k, v in LATIN_TO_CYRILLIC.items()}

    # Объединяем все DAB символы в один set для быстрой проверки
    ALL_DAB_CHARS: Final[set[str]] = (
        set(LATIN_TO_CYRILLIC.keys()) | set(LATIN_TO_CYRILLIC.values())
    )

    def __init__(self, string: str):
        self._string: str = string
        self._result: Optional[str] = None

    def main_process(self) -> None:
        """Основной процесс определения и выполнения конвертации."""
        filtered_chars = [c for c in self._string if self._is_alphabetic(c) and not self._is_dab(c)]

        # Используем генераторы для экономии памяти
        latin_score = sum(1 for c in filtered_chars if self._detect_char_alphabet(c) == 'latin')
        cyrillic_score = sum(
            1 for c in filtered_chars if self._detect_char_alphabet(c) == 'cyrillic'
        )

        self._result = (
            self._to_latin(self._string)
            if latin_score >= cyrillic_score
            else self._to_cyrillic(self._string)
        )

    @cached_property
    def checker_alphabets(self) -> list[str]:
        """Возвращает список алфавитов для каждого буквенного символа в результате."""
        return [self._detect_char_alphabet(c) for c in self._result if c.isalpha()]

    @staticmethod
    def checker_set_len_alphabets(alphabets: list[str]) -> int:
        """Возвращает количество уникальных алфавитов в списке."""
        return len(set(alphabets))

    @property
    def result(self) -> str:
        """Возвращает результат конвертации с проверкой корректности."""
        if self._result is None:
            self.main_process()

        # УБИРАЕМ ВАЛИДАЦИЮ - НЕ ВСЁ МОЖНО СКОНВЕРТИРОВАТЬ
        # if not self.checker_alphabets:  # Если нет буквенных символов
        #     return self._result
        #
        # if self.checker_set_len_alphabets(self.checker_alphabets) > 1:
        #     # err = {
        #     #     'result': self._result,
        #     #     'alphabets': self.checker_alphabets
        #     # }
        #     # pretty_print(err, title=f"err", m2d=False)
        #     # raise ValueError("Результат содержит буквы из разных алфавитов.")
        #     return self._result

        return self._result

    def _detect_char_alphabet(self, ch: str) -> str:
        """Определяет алфавит символа."""
        if '\u0041' <= ch <= '\u005A' or '\u0061' <= ch <= '\u007A':  # Unicode латиница
            return 'latin'
        elif '\u0410' <= ch <= '\u044F' or ch in ('Ё', 'ё'):  # Unicode кириллица + Ё/ё
            return 'cyrillic'
        return 'other'

    def _is_dab(self, ch: str) -> bool:
        """Проверяет, является ли символ DAB (имеет визуально похожий аналог)."""
        return ch in self.ALL_DAB_CHARS

    def _is_alphabetic(self, ch: str) -> bool:
        """Проверяет, является ли символ буквенным."""
        return ch.isalpha()

    def _count_dual_alphabet_letters(self, s: str) -> int:
        """Считает количество DAB-символов в строке."""
        return sum(1 for c in s if self._is_dab(c))

    def _to_latin(self, s: str) -> str:
        """Конвертирует все возможные символы в латиницу."""
        return s.translate(str.maketrans(self.CYRILLIC_TO_LATIN))

    def _to_cyrillic(self, s: str) -> str:
        """Конвертирует все возможные символы в кириллицу."""
        return s.translate(str.maketrans(self.LATIN_TO_CYRILLIC))

    def has_mixed_alphabet(self) -> bool:
        """
        Проверяет, содержит ли исходная строка смешанные алфавиты (латиница и кириллица)
        среди буквенных символов, игнорируя неалфавитные и не-DAB символы.

        Returns:
            bool: True, если есть символы из обоих алфавитов, иначе False.
        """
        seen_alphabets: set[str] = set()

        for char in self._string:
            if not self._is_alphabetic(char):
                continue

            alphabet = self._detect_char_alphabet(char)
            if alphabet in ('latin', 'cyrillic'):
                seen_alphabets.add(alphabet)
                # Оптимизация: если уже оба есть — можно выйти
                if len(seen_alphabets) > 1:
                    return True

        return len(seen_alphabets) == 2

    @cached_property
    def highlight(self) -> str:
        """
        Возвращает строку с выделением символов, не соответствующих доминирующему алфавиту.
        Выделение — в скобках: например, "прив(e)т".

        Использует ту же логику определения доминирующего алфавита, что и main_process.
        """
        if not self._string:
            return self._string

        # Шаг 1: определяем доминирующий алфавит (выполняем main_process, если ещё не делали)
        if self._result is None:
            self.main_process()

        # Шаг 2: получаем доминирующий алфавит из анализа чистых букв
        filtered_chars = [
            c for c in self._string
            if self._is_alphabetic(c) and not self._is_dab(c)
        ]

        latin_count = sum(1 for c in filtered_chars if self._detect_char_alphabet(c) == 'latin')
        cyrillic_count = sum(
            1 for c in filtered_chars if self._detect_char_alphabet(c) == 'cyrillic'
        )

        dominant_alphabet = 'latin' if latin_count >= cyrillic_count else 'cyrillic'

        # Шаг 3: строим строку с подсветкой
        highlighted = []
        for char in self._string:
            if self._is_alphabetic(char) and self._detect_char_alphabet(char) != dominant_alphabet:
                highlighted.append(f"({char})")  # выделяем чужие буквы
            else:
                highlighted.append(char)

        return ''.join(highlighted)



from datetime import datetime
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field


class AttemptInfo(BaseModel):
    """Информация о попытке обработки"""
    attempt_number: int
    success: bool
    error_message: Optional[str] = None
    timeout_before_next: Optional[float] = None
    timestamp: datetime


class ItemProcessingResult(BaseModel):
    """Результат обработки одного элемента"""
    item: Any
    success: bool = False  # Добавляем значение по умолчанию
    result: Optional[Any] = None
    error_message: Optional[str] = None
    attempts: List[AttemptInfo] = Field(default_factory=list)
    total_attempts: int = 0
    thread_id: Optional[int] = None


class ThreadTiming(BaseModel):
    """Модель временных меток потока"""
    start_time: Optional[datetime] = None
    end_time: Optional[datetime] = None
    duration: Optional[float] = None
    start_time_str: Optional[str] = None
    end_time_str: Optional[str] = None
    duration_str: Optional[str] = None


class ProcessorStats(BaseModel):
    """Модель статистики процессора"""
    success: int = 0
    failed: int = 0
    total: int = 0
    threads_used: int = 0
    chunk_sizes: Dict[int, int] = Field(default_factory=dict)
    thread_timings: Dict[int, ThreadTiming] = Field(default_factory=dict)
    total_start_time: Optional[datetime] = None
    total_end_time: Optional[datetime] = None
    total_duration: Optional[float] = None
    total_start_time_str: Optional[str] = None
    total_end_time_str: Optional[str] = None
    total_duration_str: Optional[str] = None
    pass_thread_id: bool = True
    supports_thread_id: bool = False
    detailed_results: Dict[str, ItemProcessingResult] = Field(default_factory=dict)
    exhausted_attempts_items: List[Any] = Field(default_factory=list)


class CompletionReport(BaseModel):
    """Отчет о завершении обработки"""
    stats: ProcessorStats
    results: List[Any]
    thread_data_dict: Dict[int, List[Any]]
    processing_time: float
    success_rate: float

    @property
    def success_percentage(self) -> str:
        """Процент успешных обработок"""
        if self.stats.total == 0:
            return "0%"
        return f"{(self.stats.success / self.stats.total) * 100:.1f}%"



from datetime import datetime
from typing import Generic, Optional, TypeAlias, TypeVar

from pydantic import BaseModel, Field


AllowedTypes: TypeAlias = str | list | int | dict

T = TypeVar('T')


class MetaState(BaseModel):
    """
    Мета-информация о состоянии
    """
    created_at: int = Field(description="Дата-время создания")
    updated_at: int = Field(description="Дата-время последнего обновления")


class DefaultState(BaseModel, Generic[T]):
    """
    Дефолтный ответ вообще для всех существующих методов.

    Attributes:
        result (Optional[bool]): Общий результат выполнения действия
        status (Optional[AllowedTypes]): Короткое воплощение details
        detail (Optional[AllowedTypes]): Текстовое сообщение об ошибке
        data (Optional[T]): Любые данные, которые должны вернуться
        meta (MetaState): Системная мета-информация (id, created_at, updated_at)
    """

    result: Optional[bool] = Field(default=False)
    status: Optional[AllowedTypes] = Field(default='INIT')
    detail: Optional[AllowedTypes] = Field(default='')
    data: Optional[T] = None
    meta: MetaState = Field(
        default_factory=lambda: MetaState(
            created_at=int(datetime.now().timestamp()),
            updated_at=int(datetime.now().timestamp())
        )
    )

    model_config = {
        "use_enum_values": True,
        "arbitrary_types_allowed": True,
    }

    def __init__(self, init_data: Optional[dict] = None, **kwargs) -> None:
        """
        Поддерживает инициализацию:
        - Из словаря: DefaultState({'result': True, ...})
        - Из ключевых аргументов: DefaultState(result=True, ...)
        - Пустую инициализацию: DefaultState()
        """
        # Создаем мета-информацию перед инициализацией
        meta = MetaState(
            created_at=int(datetime.now().timestamp()),
            updated_at=int(datetime.now().timestamp())
        )

        if init_data and isinstance(init_data, dict):
            processed_data = {
                'result': init_data.get('result', False),
                'status': init_data.get('status', 'INIT'),
                'detail': init_data.get('detail', ''),
                'data': init_data.get('data'),
            }
            # Гарантируем, что meta не перезапишется, если передано
            if 'meta' not in kwargs:
                kwargs['meta'] = self.meta
            super().__init__(**processed_data, **kwargs)
        else:
            # kwargs['meta'] = meta
            super().__init__(**kwargs)

    def __bool__(self) -> bool:
        """Позволяет использовать объект в булевом контексте (if/not)"""
        return self.result is True

    def _update_meta(self) -> None:
        """Обновляет время последнего изменения"""
        self.meta.updated_at = int(datetime.now().timestamp())

    def update(
        self: 'DefaultState[T]',
        *,
        result: bool = False,
        status: Optional[AllowedTypes] = None,
        detail: Optional[AllowedTypes] = None,
        data: Optional[T] = None,
    ) -> 'DefaultState[T]':
        """Обновляет данные стейта и мета-поля."""
        if result is not None:
            self.result = result
        if status is not None:
            self.status = status
        if detail is not None:
            self.detail = detail
        if data is not None:
            self.data = data

        self._update_meta()
        return self

    def success(
        self: 'DefaultState[T]',
        *,
        status: Optional[AllowedTypes] = None,
        detail: Optional[AllowedTypes] = None,
        data: Optional[T] = None,
    ) -> 'DefaultState[T]':
        """Устанавливает result=True и обновляет мета-поля."""
        self.result = True
        if status is not None:
            self.status = status
        if detail is not None:
            self.detail = detail
        if data is not None:
            self.data = data

        self._update_meta()
        return self

    def insert(self, data: dict) -> 'DefaultState[T]':
        """Алиас для update с передачей словаря (обратная совместимость)"""
        return self.update(**data)

    @property
    def created_at(self) -> int:
        """Короткий доступ к meta.created_at"""
        return self.meta.created_at

    @property
    def updated_at(self) -> int:
        """Короткий доступ к meta.updated_at"""
        return self.meta.updated_at

    @property
    def created_dt(self) -> datetime:
        """Возвращает created_at как объект datetime"""
        return datetime.fromtimestamp(self.created_at)

    @property
    def updated_dt(self) -> datetime:
        """Возвращает updated_at как объект datetime"""
        return datetime.fromtimestamp(self.updated_at)



import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from datetime import datetime
from threading import Lock
from typing import Any, Callable, Dict, List, Optional

from src.onco_cola_utils import log
from src.onco_cola_utils.advanced_thread_processor.constants import DefaultValues, ErrorMessages
from src.onco_cola_utils.advanced_thread_processor.models import (
    AttemptInfo, CompletionReport,
    ItemProcessingResult,
    ProcessorStats, ThreadTiming
)
from src.onco_cola_utils.advanced_thread_processor.utils import (
    MethodInspector,
    ProcessorLogger,
    TimeFormatter
)


print = log


class AdvancedThreadProcessor:
    def __init__(
        self,
        data_list: List[Any],
        process_method: Callable,
        threads_count: int = DefaultValues.THREADS_COUNT,
        max_attempts: int = DefaultValues.MAX_ATTEMPTS,
        base_timeout: float = DefaultValues.BASE_TIMEOUT,
        delta_timeout: float = DefaultValues.DELTA_TIMEOUT,
        pass_thread_id: bool = DefaultValues.PASS_THREAD_ID,
        enable_detailed_logging: bool = DefaultValues.ENABLE_DETAILED_LOGGING,
        on_complete_method: Optional[Callable] = DefaultValues.ON_COMPLETE_METHOD
    ):
        if not data_list:
            raise ValueError(ErrorMessages.EMPTY_DATA_LIST)

        self.data_list = data_list
        self.process_method = process_method
        self.threads_count = threads_count
        self.max_attempts = max_attempts
        self.base_timeout = base_timeout
        self.delta_timeout = delta_timeout
        self.pass_thread_id = pass_thread_id
        self.enable_detailed_logging = enable_detailed_logging
        self.on_complete_method = on_complete_method

        # Проверяем возможность передачи thread_id
        self._supports_thread_id = MethodInspector.supports_thread_id(process_method)

        if pass_thread_id and not self._supports_thread_id:
            raise ValueError(
                ErrorMessages.THREAD_ID_NOT_SUPPORTED.format(
                    method_name=process_method.__name__
                )
            )

        # Проверяем колбэк завершения если он указан
        if on_complete_method and not MethodInspector.supports_completion_report(
            on_complete_method
        ):
            raise ValueError(
                ErrorMessages.ON_COMPLETE_METHOD_INVALID
            )

        self.results = []
        self.stats = ProcessorStats(
            total=len(data_list),
            threads_used=threads_count,
            pass_thread_id=pass_thread_id,
            supports_thread_id=self._supports_thread_id
        )
        self._lock = Lock()
        self._thread_data_dict: Dict[int, List[Any]] = {}
        self._is_valid_threads = False

        print(f"Инициализация процессора: {len(data_list)} элементов, {threads_count} потоков")
        self._divide_by_threads()

    def _divide_by_threads(self) -> Dict[int, List[Any]]:
        """Делит данные на примерно равные части для каждого потока"""
        total_items = len(self.data_list)
        if total_items == 0:
            self._is_valid_threads = True
            return {}

        actual_threads = min(self.threads_count, total_items)
        base_chunk_size = total_items // actual_threads
        chunks_with_extra = total_items % actual_threads

        self._thread_data_dict = {}
        start_index = 0

        for thread_id in range(actual_threads):
            chunk_size = base_chunk_size + (1 if thread_id < chunks_with_extra else 0)
            end_index = start_index + chunk_size
            self._thread_data_dict[thread_id] = self.data_list[start_index:end_index]

            # Обновляем статистику через модель
            self.stats.chunk_sizes[thread_id] = chunk_size
            start_index = end_index

        self.stats.threads_used = actual_threads
        self._is_valid_threads = True

        print(f"Данные распределены по {actual_threads} потокам:")
        for thread_id, chunk in self._thread_data_dict.items():
            print(f"  Поток {thread_id}: {len(chunk)} элементов")

        return self._thread_data_dict

    def _process_with_retry(self, item, thread_id: int = None):
        """Обрабатывает один элемент с повторными попытками и детальным логированием"""
        item_key = str(item)
        processing_result = ItemProcessingResult(
            item=item,
            thread_id=thread_id,
            total_attempts=self.max_attempts
        )

        for attempt in range(self.max_attempts):
            try:
                if self.pass_thread_id and self._supports_thread_id and thread_id is not None:
                    result = self.process_method(item, thread_id=thread_id)
                else:
                    result = self.process_method(item)

                # Успешная попытка
                processing_result.success = True
                processing_result.result = result
                processing_result.attempts.append(
                    AttemptInfo(
                        attempt_number=attempt + 1,
                        success=True,
                        timestamp=datetime.now()
                    )
                )

                if self.enable_detailed_logging:
                    ProcessorLogger.log_attempt(
                        thread_id=thread_id or 0,
                        item=item,
                        attempt=attempt + 1,
                        max_attempts=self.max_attempts,
                        success=True
                    )

                with self._lock:
                    self.stats.success += 1
                    self.stats.detailed_results[item_key] = processing_result

                return result

            except Exception as e:
                error_msg = str(e)
                timeout = self.base_timeout + (attempt * self.delta_timeout)

                # Записываем информацию о неудачной попытке
                processing_result.attempts.append(
                    AttemptInfo(
                        attempt_number=attempt + 1,
                        success=False,
                        error_message=error_msg,
                        timeout_before_next=timeout if attempt < self.max_attempts - 1 else None,
                        timestamp=datetime.now()
                    )
                )

                if self.enable_detailed_logging:
                    ProcessorLogger.log_attempt(
                        thread_id=thread_id or 0,
                        item=item,
                        attempt=attempt + 1,
                        max_attempts=self.max_attempts,
                        success=False,
                        error_msg=error_msg,
                        timeout=timeout
                    )

                # Если это последняя попытка
                if attempt == self.max_attempts - 1:
                    processing_result.success = False
                    processing_result.error_message = error_msg

                    with self._lock:
                        self.stats.failed += 1
                        self.stats.detailed_results[item_key] = processing_result
                        self.stats.exhausted_attempts_items.append(item)

                    raise e

                # Ждем перед следующей попыткой
                time.sleep(timeout)

    def _process_thread_chunk(self, thread_id: int):
        """Обрабатывает чанк данных для одного потока"""
        print(f"Запуск потока {thread_id} с {len(self._thread_data_dict[thread_id])} элементами")

        thread_start_time = datetime.now()

        # Инициализируем timing для потока
        with self._lock:
            self.stats.thread_timings[thread_id] = ThreadTiming(
                start_time=thread_start_time
            )

        thread_data = self._thread_data_dict.get(thread_id, [])
        thread_results = []

        for i, item in enumerate(thread_data):
            if i % 100 == 0:  # Логируем прогресс каждые 100 элементов
                print(f"Поток {thread_id}: обработано {i}/{len(thread_data)} элементов")

            try:
                result = self._process_with_retry(item, thread_id)
                thread_results.append(result)
            except Exception as e:
                # Ошибка уже обработана в _process_with_retry
                print(
                    f"Поток {thread_id}: элемент {item} завершился с ошибкой после всех попыток: {e}"
                )
                pass

        print(
            f"Поток {thread_id} завершил работу. Обработано: {len(thread_results)}/{len(thread_data)}"
        )

        thread_end_time = datetime.now()
        thread_duration = (thread_end_time - thread_start_time).total_seconds()

        # Обновляем timing с форматированными строками
        with self._lock:
            self.stats.thread_timings[thread_id] = ThreadTiming(
                start_time=thread_start_time,
                end_time=thread_end_time,
                duration=thread_duration,
                start_time_str=TimeFormatter.format_datetime(thread_start_time),
                end_time_str=TimeFormatter.format_datetime(thread_end_time),
                duration_str=TimeFormatter.format_duration(thread_duration)
            )

        return thread_results

    def _call_completion_callback(self, processing_time: float):
        """Вызывает колбэк завершения если он указан"""
        if not self.on_complete_method:
            return

        try:
            # Создаем отчет о завершении
            success_rate = self.stats.success / self.stats.total if self.stats.total > 0 else 0
            report = CompletionReport(
                stats=self.statistics,
                results=self.results.copy(),
                thread_data_dict=self.thread_data_dict,
                processing_time=processing_time,
                success_rate=success_rate
            )

            # Вызываем колбэк
            self.on_complete_method(report=report)

        except Exception as e:
            print(f"Ошибка при вызове колбэка завершения: {e}")

    def run(self):
        """Запускает многопоточную обработку"""
        if not self._is_valid_threads:
            raise ValueError(ErrorMessages.DATA_NOT_PREPARED)

        if not self._thread_data_dict:
            print("Нет данных для обработки")
            return self.results

        print(f"Запуск обработки {len(self.data_list)} элементов в {self.threads_count} потоках")
        self.stats.total_start_time = datetime.now()

        try:
            with ThreadPoolExecutor(max_workers=self.threads_count) as executor:
                future_to_thread = {
                    executor.submit(self._process_thread_chunk, thread_id): thread_id
                    for thread_id in self._thread_data_dict.keys()
                }

                for future in as_completed(future_to_thread):
                    try:
                        thread_results = future.result()
                        self.results.extend(thread_results)
                        print(
                            f"Поток {future_to_thread[future]} завершился успешно. Результатов: {len(thread_results)}"
                        )
                    except Exception as e:
                        print(f"Критическая ошибка в потоке {future_to_thread[future]}: {e}")

        except Exception as e:
            print(f"Ошибка при выполнении потоков: {e}")
            raise

        self.stats.total_end_time = datetime.now()
        total_duration = (self.stats.total_end_time - self.stats.total_start_time).total_seconds()

        # Обновляем общую статистику с форматированными строками
        self.stats.total_duration = total_duration
        self.stats.total_start_time_str = TimeFormatter.format_datetime(self.stats.total_start_time)
        self.stats.total_end_time_str = TimeFormatter.format_datetime(self.stats.total_end_time)
        self.stats.total_duration_str = TimeFormatter.format_duration(total_duration)

        print(f"Обработка завершена. Всего результатов: {len(self.results)}")

        # Вызываем колбэк завершения
        self._call_completion_callback(total_duration)

        return self.results

    @property
    def thread_data_dict(self) -> Dict[int, List[Any]]:
        return self._thread_data_dict.copy()

    @property
    def statistics(self) -> ProcessorStats:
        """Возвращает статистику (копию для безопасности)"""
        return self.stats.model_copy()

    @property
    def is_ready(self) -> bool:
        return self._is_valid_threads and bool(self._thread_data_dict)

    def get_failed_items_report(self) -> List[Dict]:
        """Возвращает отчет по неудачно обработанным элементам"""
        report = []
        for item_key, result in self.stats.detailed_results.items():
            if not result.success:
                report.append(
                    {
                        'item': result.item,
                        'thread_id': result.thread_id,
                        'error_message': result.error_message,
                        'attempts_made': len(result.attempts),
                        'max_attempts': result.total_attempts,
                        'last_attempt_time': (
                            result.attempts[-1].timestamp if result.attempts else None
                        )
                    }
                )
        return report



import inspect
from datetime import datetime, timedelta
from typing import Any

from src.onco_cola_utils.advanced_thread_processor.models import CompletionReport
from src.onco_cola_utils.advanced_thread_processor.constants import TimeFormats


class TimeFormatter:
    """Утилиты для форматирования времени"""

    @staticmethod
    def format_duration(seconds: float) -> str:
        """Форматирует время в формате 'ДДД дн. ЧЧ ч. ММ мин. СС сек.'"""
        if seconds <= 0:
            return "0 сек."

        td = timedelta(seconds=seconds)
        days = td.days
        hours, remainder = divmod(td.seconds, 3600)
        minutes, seconds = divmod(remainder, 60)

        parts = []
        if days > 0:
            parts.append(f"{days} дн.")
        if hours > 0:
            parts.append(f"{hours:02d} ч.")
        if minutes > 0:
            parts.append(f"{minutes:02d} мин.")
        if seconds > 0 or not parts:
            parts.append(f"{seconds:02d} сек.")

        return " ".join(parts)

    @staticmethod
    def format_datetime(dt) -> str:
        """Форматирует datetime в строку"""
        if not dt:
            return ""
        return dt.strftime(TimeFormats.DATETIME_DISPLAY)


class MethodInspector:
    """Утилиты для анализа методов"""

    @staticmethod
    def supports_thread_id(method) -> bool:
        """
        Проверяет, поддерживает ли метод параметр thread_id: int
        """
        import inspect

        try:
            sig = inspect.signature(method)

            if 'thread_id' in sig.parameters:
                param = sig.parameters['thread_id']
                # Проверяем аннотацию типа
                if param.annotation in (int, inspect.Parameter.empty):
                    return True
                # Если аннотация указана, проверяем что это int
                elif hasattr(param.annotation, '__origin__'):
                    return param.annotation.__origin__ is int
                else:
                    return param.annotation is int

            return False

        except (ValueError, TypeError):
            return False

    @staticmethod
    def supports_completion_report(method) -> bool:
        """
        Проверяет, поддерживает ли метод параметр report: CompletionReport
        """
        try:
            sig = inspect.signature(method)

            if 'report' in sig.parameters:
                param = sig.parameters['report']
                # Проверяем аннотацию типа
                if param.annotation in (CompletionReport, inspect.Parameter.empty):
                    return True
                # Если аннотация указана, проверяем что это CompletionReport
                elif hasattr(param.annotation, '__origin__'):
                    return param.annotation.__origin__ is CompletionReport
                else:
                    return param.annotation is CompletionReport

            return False

        except (ValueError, TypeError):
            return False


class ProcessorLogger:
    """Логгер для процессора"""

    @staticmethod
    def log_attempt(
        thread_id: int, item: Any, attempt: int, max_attempts: int,
        success: bool, error_msg: str = None, timeout: float = None
    ):
        """Логирует информацию о попытке"""
        timestamp = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        status = "УСПЕШНО" if success else "ОШИБКА"

        if success:
            print(
                f"{timestamp} - [TH={thread_id}] Попытка {attempt}/{max_attempts}: {status} - {item}"
            )
        else:
            if attempt < max_attempts:
                print(
                    f"{timestamp} - [TH={thread_id}] Попытка {attempt}/{max_attempts}: {status} - {item} | "
                    f"Ошибка: {error_msg} | Следующая попытка через: {timeout:.1f} сек."
                )
            else:
                print(
                    f"{timestamp} - [TH={thread_id}] Попытка {attempt}/{max_attempts}: {status} - {item} | "
                    f"Ошибка: {error_msg} | ВСЕ ПОПЫТКИ ИСЧЕРПАНЫ!"
                )



"""КАСТОМНЫЕ ОШИБКИ"""


class ContentLengthError(Exception):
    """Нет содержимого"""
    pass


class WrongSheetListError(Exception):
    """Неверный лист документа"""
    pass


class LocalIDError(Exception):
    """Отсутствует local_id"""
    pass


class EmptyIdfyNotNullDictException(Exception):
    """Perfect-данные пусты"""
    pass



import sys
from typing import Optional

from ..logger.core import logerr, logsuc


def halt(msg: Optional[str] = "Принудительный выход"):
    """Грубое прерывание хода программы"""
    logerr("")
    logerr(msg)
    logerr("")
    sys.exit(0)


def quiet(msg: Optional[str] = "Успешное завершение программы") -> None:
    """Тихое завершение кода программы"""
    logsuc("")
    logsuc(msg)
    logsuc("")
    sys.exit(0)



from .configs.column_strings import ColumnStrings
from .configs.system import System
from .logger.core import *
from .pretty_print.core import pretty_print
from .reader_controller.core import ReaderController
from .value_to_bool import value_to_bool



"""ЯДРО"""

import sys

from loguru import logger


logger.remove()

logger.level("WARNING", color="<light-yellow>")  # светло-жёлтый (более яркий)

# Обработчики для всех уровней
for level in ["DEBUG", "INFO", "SUCCESS", "WARNING", "ERROR"]:
    logger.add(
        sys.stderr,
        format="<green>{time:YYYY-MM-DD HH:mm:ss}</green> - <level>{message}</level>",
        level=level,
        filter=lambda record, lvl=level: record["level"].name == lvl,
        colorize=True
    )

# Алиасы
log = logger.debug
loginf = logger.info
logsuc = logger.success
logwarn = logger.warning
logerr = logger.error


from typing import Callable, Optional


class TimeFormats:
    """Форматы времени"""
    DATETIME_DISPLAY = '%Y-%m-%d %H:%M:%S'
    DATE_ONLY = '%Y-%m-%d'
    TIME_ONLY = '%H:%M:%S'


class DefaultValues:
    """Значения по умолчанию"""
    THREADS_COUNT = 5
    MAX_ATTEMPTS = 1
    BASE_TIMEOUT = 5.0
    DELTA_TIMEOUT = 2.0
    PASS_THREAD_ID = True
    ENABLE_DETAILED_LOGGING = True
    ON_COMPLETE_METHOD: Optional[Callable] = None


class ErrorMessages:
    """Сообщения об ошибках"""
    DATA_NOT_PREPARED = "Данные для потоков не подготовлены"
    THREAD_ID_NOT_SUPPORTED = "Метод {method_name} не поддерживает параметр thread_id: int. Используйте pass_thread_id=False или добавьте параметр thread_id: int в метод."
    ON_COMPLETE_METHOD_INVALID = "Метод on_complete_method должен принимать параметр report: CompletionReport"
    EMPTY_DATA_LIST = "Список данных не может быть пустым"



import time
from datetime import timedelta
from typing import Optional

from ..logger import log, loginf


class ProgressMessage:
    """
    Генератор строки прогресса с метриками: процент завершения, скорость обработки,
    прошедшее и оставшееся время. Поддерживает управляемый лог через log_me().
    """
    
    def __init__(
        self,
        total: int,
        processed: int = 0,
        start_time: Optional[float] = None,
        every: Optional[timedelta] = None,
    ) -> None:
        if total <= 0:
            raise ValueError("Общее количество должно быть положительным целым числом")
        self.total = total
        self.processed = max(0, processed)
        self.start_time = start_time or time.time()
        if not isinstance(every, timedelta):
            raise TypeError("Объект every должен быть timedelta с указанием периодичности")
        self._every_seconds = every.total_seconds() if every is not None else None
        self._last_log_time = self.start_time
        
        # Логируем режим работы
        if self._every_seconds is None:
            loginf("Логирование в режиме каждого вызова")
        else:
            loginf(f"Логирование в режиме периодического вызова ({self._every_seconds:.1f} сек.)")
    
    def update(self, processed: int) -> None:
        self.processed = max(0, min(processed, self.total))
    
    def _duration_str(self, seconds: float) -> str:
        if not isinstance(seconds, (int, float)) or seconds != seconds:  # NaN
            return "∞"
        if seconds == float('inf'):
            return "∞"
        total_seconds = int(seconds)
        h, rem = divmod(total_seconds, 3600)
        m, s = divmod(rem, 60)
        return f"{h:02d}ч{m:02d}м{s:02d}с"
    
    @property
    def msg(self) -> str:
        elapsed = time.time() - self.start_time
        if elapsed <= 0:
            speed = 0.0
            remaining_time = float('inf')
        else:
            speed = self.processed / elapsed
            remaining = self.total - self.processed
            remaining_time = remaining / speed if speed > 0 else float('inf')
        
        percent = (self.processed / self.total) * 100
        remain = self.total - self.processed
        
        percent_str = f"{percent:5.2f}%"
        spd_str = f"{speed:6.3f}"
        et_str = self._duration_str(elapsed)
        rt_str = self._duration_str(remaining_time)
        
        return f" || {self.processed}/{self.total}//{remain} || {percent_str} || IPS: {spd_str} || ET: {et_str} || RT: {rt_str}"
    
    def log_me(self, template: str = "{log}") -> None:
        """
        Логирует сообщение прогресса по заданному шаблону.

        Подстрока `{log}` заменяется на актуальное значение self.msg.
        Логирование происходит:
          - всегда, если processed == total (финальное сообщение),
          - иначе — только если прошло достаточно времени (согласно `every`).

        :param template: Строка, содержащая `{log}`. По умолчанию — "{log}".
        """
        now = time.time()
        is_complete = (self.processed >= self.total)
        
        should_log = (
            is_complete  # Финальное сообщение — обязательно!
            or self._every_seconds is None  # Логировать каждый раз
            or (now - self._last_log_time) >= self._every_seconds  # Прошёл интервал
        )
        
        if should_log:
            message = template.replace("{log}", self.msg)
            log(message)
            self._last_log_time = now



import traceback
from pathlib import Path


class FileReaderController:
    _encoding = "utf-8"

    @classmethod
    def save_text(cls, file_path: Path, content: str):
        """
        Безопасно записывает текст в файл с указанной кодировкой.
        """
        try:
            file_path.parent.mkdir(parents=True, exist_ok=True)  # создаем директорию если нужно
            file_path.write_text(content, encoding=cls._encoding)
        except Exception as e:
            traceback.print_exc()
            return False
        else:
            return True

    @classmethod
    def read_text(cls, file_path: Path):
        """
        Чтение файла
        """

        try:
            content = file_path.read_text(encoding=cls._encoding)
        except Exception as e:
            traceback.print_exc()
            return False
        else:
            return content
